unit USPClient;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ScktComp, USPUtils;

type

  TUSPClient = class(TComponent)
  private
    FPort: integer;
    FClient: TClientSocket;
    FTimeout: integer;
    FLogin: String;
    FPassword: String;
    //FLastPacket: String;
    //FCount: integer;
    FOnError: TUSPError;
    FProtocol: string;
    function GetHost: String;
    procedure SetHost(Host:String);
  protected
    procedure ClientRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ClientError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    function ReadPacket:string;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function CmdExec(cmd,dot:string):string;
    function SrvCall(cmd:string;Params: array of const):TPacket;
    procedure SetPort(Num: integer);
  published
    property Port: integer read FPort write SetPort;
    property ServHost: String read GetHost write SetHost;
    property Timeout: Integer read FTimeout write FTimeout;
    property UserLogin: String read FLogin write FLogin;
    property UserPassword: String read FPassword write FPassword;
    property OnError: TUSPError read FOnError write FOnError;
    property Protocol: string read FProtocol write FProtocol;
    //property OnChangeState: TODChangeState read FOnChangeState write FOnChangeState;
    function Open:bool;
    procedure Close;
  end;

procedure Register;

implementation

{ TUSPClient }

function TUSPClient.ReadPacket:string;
var T: TSystemTime; w: integer;
    FEndPacket: bool;
begin
  Result:='';
  FEndPacket:=false;
  GetLocalTime(T);
  w:=t.wMinute*60+t.wSecond;
  repeat
    Application.ProcessMessages;
    GetLocalTime(T);
    if (t.wMinute*60+t.wSecond-w)>FTimeout then begin
      if Assigned(FOnError) then FOnError('Connection timeout ('+IntToStr(FTimeout)+' seconds)');
      FEndPacket:=true;
    end;
    if FClient.Socket.ReceiveLength>0 then FEndPacket:=true;
  until FEndPacket;
  Result:=FClient.Socket.ReceiveText;
end;

function TUSPClient.CmdExec(cmd,dot:string):string;
var s,ed:string;
begin
  Result:='';
  if FClient.Active then begin
    FClient.Socket.SendText(cmd+#13#10);
    if dot='' then Result:=ReadPacket
    else begin
      repeat
        s:=ReadPacket;
        Result:=Result+s;
        ed:=Copy(Result,length(Result)-length(dot)+1,length(dot));
      until (ed=dot)or(s='');
      if Result[length(Result)]=#10
      then SetLength(s,length(s)-2);
    end;
  end else if Assigned(FOnError) then FOnError('Client is not connected');
end;

function TUSPClient.SrvCall(cmd:string; Params: array of const):TPacket;
var Par: TVarRec; i:integer; ps:string;
begin
  ps:='';
  for i:=0 to length(Params)-1 do begin
    Par:=Params[i];
    case Par.VType of
      vtInteger:    ps:=ps+IntToStr(Par.VInteger);
      vtBoolean:    ps:=ps+IntToStr(Ord(Par.VBoolean));
      vtChar:       ps:=ps+Par.VChar;
      vtExtended:   ps:=ps+FloatToStr(Par.VExtended^);
      vtString:     ps:=ps+PChar(Par.VString);
      vtPChar:      ps:=ps+Par.VPChar;
      vtWideChar:   ps:=ps+Par.VWideChar;
      vtAnsiString: ps:=ps+Pchar(Par.VAnsiString);
      vtWideString: ps:=ps+PChar(Par.VWideString);
      vtObject:begin
        ps:=ps+Par.VObject.ClassName;
      end;
    end;
    ps:=ps+',';
  end;
  SetLength(ps,length(ps)-1);
  ps:=cmd+'('+ps+').';
  Result:=TPacket.Create(CmdExec(ps,PacketEnd));
end;

constructor TUSPClient.Create(AOwner: TComponent);
begin
  FClient:=TClientSocket.Create(Self);
  inherited Create(AOwner);
  FClient.Port:=2032;
  FClient.OnConnect:=ClientConnect;
  FClient.OnRead:=ClientRead;
  FClient.OnError:=ClientError;
  FTimeout:=30;
end;

destructor TUSPClient.Destroy;
begin
  Close;
  FClient.Free;
  inherited  Destroy;
end;

function TUSPClient.GetHost: String;
begin
  Result:=FClient.Host;
end;

procedure TUSPClient.SetHost(Host:String);
begin
  if (not FClient.Active)and(FClient.Host<>Host)
  then FClient.Host:=Host;
end;

procedure TUSPClient.SetPort(Num:integer);
begin
  if (not FClient.Active)and(FPort<>Num) then begin
    FPort:=Num;
    FClient.Port:=Num;
  end;
end;

function TUSPClient.Open:bool;
var T: TSystemTime; w:integer;
begin
  Close;
  FClient.Open;
  GetLocalTime(T);
  w:=t.wMinute*60+t.wSecond;
  repeat
    Application.ProcessMessages;
    GetLocalTime(T);
  until (FClient.Active)or((t.wMinute*60+t.wSecond-w)>FTimeout);
  Result:=FClient.Active
end;

procedure TUSPClient.Close;
begin
  if FClient.Active then FClient.Close;
end;

procedure TUSPClient.ClientRead(Sender: TObject; Socket: TCustomWinSocket);
{var TEMP: TStrings;
    i,k:integer;}
begin
{  TEMP:=TStringList.Create;
  FLastPacket:=Socket.ReceiveText;
  k:=Length(FLastPacket);
  if (k>1)and(FLastPacket[k-1]=#13)
  then SetLength(FLastPacket,k-2);
  TEMP.Text:=FLastPacket;
  for i:=0 to TEMP.Count-1 do ComDo(TEMP[i]);
  TEMP.Free;
  FEndPacket:=true;}
end;

procedure TUSPClient.ClientConnect(Sender: TObject; Socket: TCustomWinSocket);
begin
  ReadPacket;
  if (FLogin<>'')and(FPassword<>'')
  then SrvCall('SelectService',[Protocol,FLogin,FPassword]);
end;

procedure TUSPClient.ClientError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
var s:string;
begin
  s:='Phisical socket error # '+IntToStr(ErrorCode);
  if Assigned(FOnError) then FOnError(s)
  else Err(s);
  ErrorCode:=0;
end;

// Register

procedure Register;
begin
  RegisterComponents('USP',[TUSPClient]);
end;

end.
