unit USPServer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ScktComp, USPUtils, USPFunction;

type

  {Component}

  TUSPServer = class(TComponent)
  private
    FPort: integer;
    FServer: TServerSocket;
    FTimeout: integer;
    FOnError: TUSPError;
    FOnLogon: TUSPOnLogon;
    FOnConnect: TUSPOnConnect;
    FOnDisconnect: TUSPOnDisconnect;
    FOwner: string;
    FServices: string;
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetPort(Num: integer);
    procedure Log(s: string);
    function InIPList(s:string):bool;
    //procedure ProcessPacket(User: TUSPUser);
    // Sockets
    procedure ServerAccept(Sender: TObject; Socket: TCustomWinSocket);
    procedure ServerClientConnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ServerClientDisconnect(Sender: TObject; Socket: TCustomWinSocket);
    procedure ServerClientError(Sender: TObject; Socket: TCustomWinSocket; ErrorEvent: TErrorEvent; var ErrorCode: Integer);
    procedure ServerClientRead(Sender: TObject; Socket: TCustomWinSocket);
  published
    property Port: integer read FPort write SetPort;
    property Timeout: Integer read FTimeout write FTimeout;
    property OnError: TUSPError read FOnError write FOnError;
    property OnLogon: TUSPOnLogon read FOnLogon write FOnLogon;
    property OnConnect: TUSPOnConnect read FOnConnect write FOnConnect;
    property OnDisconnect: TUSPOnDisconnect read FOnDisconnect write FOnDisconnect;
    property ServerOwner: string read FOwner write FOwner;
    property Services: string read FServices write FServices;
    procedure Open;
    procedure Close;
  end;

  TUSPThread = class(TThread)
  private
    FProcess:Boolean;
    FUser:TUSPUser;
    FSrv:TUSPServer;
  protected
    procedure Execute; override;
    procedure ProcessPacket;
  published
    property Process:Boolean read FProcess write FProcess;
    property User:TUSPUser read FUser write FUser;
    property Srv:TUSPServer read FSrv write FSrv;
  end;

procedure Register;
function IsUserInfoValid(name,pwd,port,proto:string):bool;

var
  CurDir: string;
  Users: TUSPUsers;
  IPList,Ini,Acc: TStrings;

implementation

{ Accounts Support }

//function GetUserGroup(name:string):string;

function IsUserInfoValid(name,pwd,port,proto:string):bool;
var t,s,g:string;
begin
  Result:=false;
  t:=Acc.Values['User['+Name+']'];
  if (t<>'')and(GetStrField('Pwd',t)=pwd) then begin
    s:=Acc.Values['Group['+GetStrField('Group',t)+']'];
    g:=GetStrField('Ports',s);
    t:=GetStrField('Protocols',s);
    if (IsInList(port,g))and(IsInList(proto,t))
    then Result:=true;
  end;
end;

{TUSPThread}

procedure TUSPThread.Execute;
begin
  FProcess:=true;
  if (FUser<>nil)and(FSrv<>nil) then begin
    try
      ProcessPacket;
    except
      User.Socket.SendText('Res[Error_RPC_Exec]'+PacketEnd);
      Srv.Log(FUser.IP+'Error exec RPC procedure');
    end;
  end;
  FProcess:=false;
end;

procedure TUSPThread.ProcessPacket;
var Par: TStrings; s,cmd,d:string;
    pktlen: integer;
    Func: TUSPFunction;
    Pack: string;
begin
  Pack:=User.PacketBuffer; User.PacketBuffer:='';
  Par:=TStringList.Create; d:=',';
  s:=GetStrLine(#13#10,Pack);
  if Pos('(',s)=0 then begin
    if Pos(' ',s)=0 then s:=s+'()'
    else begin
      s[Pos(' ',s)]:='('; s:=s+')';
      d:=' ';
    end;
  end;
  if Pos('.',s)>0 then Delete(s,Pos('.',s),1); s:=s+'.';
  cmd:=Trim(GetStrLine('(',s));
  s:=GetStrLine(')',s);
  pktlen:=0;
  if cmd<>'' then begin
    if s<>'' then begin
      s:=s+d;
      while (pktlen<>length(s))and(Trim(s)<>'') do begin
        pktlen:=length(s);
        Par.Add(GetStrLine(d,s));
      end;
    end;
    if (Par.Count=1)and(Par[0]='$$') then Par.Text:=Pack;
    if (lowercase(cmd)='selectservice')or(lowercase(cmd)='sls') then begin
      s:=ResOK;
      if Par.Count=3 then begin
        if not Assigned(Srv.OnLogon) then begin
          if IsUserInfoValid(Par[1],PAr[2],IntToStr(User.Socket.LocalPort),Par[0]) then begin
            if IsInList(Par[0],Srv.Services) then begin
              User.Service:=Par[0]; User.UserName:=Par[1]; User.Password:=Par[2];
              User.ServiceModule:=TDataModule(Application.FindComponent(User.Service));
            end;
            if User.ServiceModule=nil then SetStrField('Res','ERR_NoService',s);
          end else SetStrField('Res','ERR_AccessDenied',s);
        end else begin
          User.Service:=Par[0]; User.UserName:=Par[1]; User.Password:=Par[2];
          User.ServiceModule:=TDataModule(Application.FindComponent(User.Service));
          if User.ServiceModule=nil then SetStrField('Res','ERR_NoService',s);
          Srv.OnLogon(User);
        end;
      end else SetStrField('Res','ERR_SelectingService',s);
      Inc(User.Outgoing,length(s)+3);
      User.Socket.SendText(s+PacketEnd);
      cmd:='';
    end;
    if lowercase(cmd)='man' then begin
      s:=SmallPacket;
      if User.ServiceModule<>nil then begin
        Func:=TUSPFunction(User.ServiceModule.FindComponent(par[0]));
        if Func<>nil then if not(Func.ClassName='TUSPFunction') then Func:=nil;
        if Func<>nil then s:=s+'Man['+FieldEncode(Func.Help)+']';
      end;
      Inc(User.Outgoing,length(s)+3);
      User.Socket.SendText(s+PacketEnd);
      cmd:='';
    end;
    if cmd<>'' then begin
      s:=SmallPacket;
      if User.ServiceModule<>nil then begin
        Func:=TUSPFunction(User.ServiceModule.FindComponent(cmd));
        if Func<>nil then if not(Func.ClassName='TUSPFunction') then Func:=nil;
        if Func<>nil then begin
          Func.SetParameters(Par.Text);
          if Func.ParCount=Par.Count then begin
            Func.Call(User);
            s:=Func.USPResult.Text;
            Setlength(s,length(s)-2);
          end else SetStrField('Res','ERR_InvalidParameters',s);
        end else SetStrField('Res','ERR_NoFunction',s);
      end else SetStrField('Res','ERR_NoServiceSelected',s);
      Inc(User.Outgoing,length(s)+3);
      User.Socket.SendText(s+PacketEnd);
    end;
  end;
  Par.Free;
end;

{ TUSPServer }

constructor TUSPServer.Create(AOwner: TComponent);
begin
  FServer:=TServerSocket.Create(Self);
  inherited Create(AOwner);
  FServer.Port:=2032;
  Log('*** Service started');
  if (not (csDesigning in ComponentState))and(ini.Count=0) then begin
    if FileExists(CurDir+'valid_ip.lst')
    then IPList.LoadFromFile(CurDir+'valid_ip.lst');
    if FileExists(CurDir+'USP.ini')
    then INI.LoadFromFile(CurDir+'USP.ini');
    if FileExists(CurDir+'Accounts.ini')
    then Acc.LoadFromFile(CurDir+'Accounts.ini');
  end;
  // Sockets
  FServer.OnAccept:=ServerAccept;
  FServer.OnClientConnect:=ServerClientConnect;
  FServer.OnClientDisconnect:=ServerClientDisconnect;
  FServer.OnClientError:=ServerClientError;
  FServer.OnClientRead:=ServerClientRead;
end;

destructor TUSPServer.Destroy;
begin
  Close;
  FServer.Free;
  inherited  Destroy;
end;

procedure TUSPServer.Log(s:string);
var OutStream: TStream;
    FN:string;
begin
  if not (csDesigning in ComponentState) then begin
    FN:=CurDir+'USP.log';
    try
      if FileExists(FN)
      then OutStream:=TFileStream.Create(FN,fmOpenReadWrite)
      else OutStream:=TFileStream.Create(FN,fmCreate);
      OutStream.Position:=OutStream.Size;
      s:=DateToStr(Date)+'  '+TimeToStr(Time)+'  '+s+#13#10;
      OutStream.Write(s[1],length(s));
    finally
      if OutStream<>nil then OutStream.Free;
    end;
  end;
end;

function TUSPServer.InIPList(s:string):bool;
var i:integer;
begin
  Result:=false;
  if IPList.IndexOf(s)>-1 then Result:=true;
  if not Result then begin
    for i:=0 to IPList.Count-1 do begin
      if IsLike(IPList.Strings[i],s)
      then begin Result:=True; Exit; end;
    end;
  end;
end;

procedure TUSPServer.SetPort(Num:integer);
begin
  if (not FServer.Active)and(FPort<>Num) then begin
    FPort:=Num;
    FServer.Port:=Num;
  end;
end;

procedure TUSPServer.Open;
begin
  Close;
  FServer.Open;
end;

procedure TUSPServer.Close;
begin
  if FServer.Active then FServer.Close;
end;

procedure TUSPServer.ServerAccept(Sender: TObject; Socket: TCustomWinSocket);
var s:string;
begin
  if not(InIPList(Socket.RemoteAddress)) then begin
    s:=SmallPacket;
    SetStrField('Res','ERR_NotInIpList',s);
    Socket.SendText(s+PacketEnd);
    Socket.Close;
  end;
end;

procedure TUSPServer.ServerClientConnect(Sender: TObject; Socket: TCustomWinSocket);
var s:string; User: TUSPUser;
begin
  Log(Socket.RemoteAddress+'  connected');
  User:=TUSPUser.Create;
  Users.Add(User);
  User.Socket:=Socket;
  s:=SmallPacket;
  SetStrField('Servername','First USP Server',s);
  SetStrField('Owner',FOwner,s);
  s:=s+#13#10+'Time['+DateToStr(Date)+'] Date['+TimeToStr(Time)+'] Services['+FServices+']';
  Socket.SendText(s+PacketEnd);
  if Assigned(FOnConnect) then FOnConnect(User);
end;

procedure TUSPServer.ServerClientDisconnect(Sender: TObject; Socket: TCustomWinSocket);
var i:integer;
begin
  Log(Socket.RemoteAddress+'  disconnected');
  i:=0;
  while i<=Users.Count-1 do begin
    if Users[i].Socket=Socket then begin
      if Assigned(FOnDisconnect) then FOnDisconnect(Users[i]);
      Users.Delete(i);
    end;
    inc(i);
  end;
end;

procedure TUSPServer.ServerClientError(Sender: TObject;
  Socket: TCustomWinSocket; ErrorEvent: TErrorEvent;
  var ErrorCode: Integer);
begin
  Log(Socket.RemoteAddress+'  Error #'+IntToStr(ErrorCode));
  ErrorCode:=0;
end;

procedure TUSPServer.ServerClientRead(Sender: TObject; Socket: TCustomWinSocket);
var i:integer; s,t:string; User: TUSPUser;
    UT:TUSPThread;
begin
  i:=0; User:=nil;
  while i<=Users.Count-1 do begin
    if Users[i].Socket=Socket
    then User:=Users[i];
    inc(i);
  end;
  if User<>nil then begin
    s:=Socket.ReceiveText;
    if (length(s)=1)and(Ord(s[1])=8)
    then SetLength(User.PacketBuffer,Length(User.PacketBuffer)-1)
    else begin
      Inc(User.Incoming,length(s));
      User.PacketBuffer:=User.PacketBuffer+s;
      if length(User.PacketBuffer)>3
      then t:=Copy(User.PacketBuffer,length(User.PacketBuffer)-2,3)
      else t:='';
      if t=PacketEnd then begin
        UT:=TUSPThread.Create(true);
        UT.FreeOnTerminate:=true;
        UT.Priority:=tpNormal;
        UT.User:=User;
        UT.Srv:=Self;
        UT.Resume;
      end;
    end;
  end else Socket.Close;
end;

// Register

procedure Register;
begin
  RegisterComponents('USP',[TUSPServer]);
end;

initialization
  CurDir:=SPath(ExtractFilePath(ParamStr(0)));
  Users:=TUSPUsers.Create;
  IPList:=TStringList.Create;
  Ini:=TStringList.Create;
  Acc:=TStringList.Create;

finalization
  Users.Free;
  IPList.Free;
  Ini.Free;
  Acc.Free;

end.
