unit USPUtils;

{******************************************************}
{*     USP         *}
{* H: 3..2000 p               *}
{* Shaman[XNP], 2:463/482.88@Fido, 2000:12/88@X-Net   *}
{* e-mail: sth@zhadum.alfacom.net, sth_xnp@yahoo.com  *}
{******************************************************}

interface

uses SysUtils, Classes, windows, ScktComp, Forms, Dialogs, ComCtrls;

type

  TPacketList = class;
  TUSPUser = class;

  TPacketType = (ptPlain,ptMultipart,ptArray,ptFile,ptTable);
  TPacketModel = (pmSmall,pmLarge);
  TUSPError = procedure(ErrorMessage: String) of object;
  TUSPCall = procedure(User: TUSPUser; Par:TStrings; var Result:TStrings) of object;
  TUSPOnLogon = procedure(User: TUSPUser) of object;
  TUSPOnConnect = TUSPOnLogon;
  TUSPOnDisconnect = TUSPOnLogon;

  TPacket = class(TObject)
  private
    FText: string;
    FFields: string;
    FPacketName: string;
    FType: TPacketType;
    FPacketModel: TPacketModel;
    FReplyOK: bool;
    FErrorCode: string;
    FSubCount: integer;
    FSubPackets: TPacketList;
  protected
    procedure DecodePacket;
    function GetField(name: string):string;
    procedure SetField(name,value: string);
    function GetCell(row,col:integer):string;
    procedure SetCell(row,col:integer;val:string);
    function GetFieldCell(row:integer;field:string):string;
    procedure SetFieldCell(row:integer;field,val:string);
    function FindSub(name: string):TPacket;
    function ClipText:string;
  public
    constructor Create(pText: string);
    destructor Destroy; override;
    procedure CreateSubPacket(pText: string);
    property Text: string read ClipText write FFields;
    property Fields[name:string]:string read GetField write SetField;
    property ArrayCell[row,col:integer]:string read GetCell write SetCell;
    property PacketName: string read FPacketName;
    property PacketType: TPacketType read FType;
    property Model: TPacketModel read FPacketModel;
    property SubCount: integer read FSubCount;
    property Sub[name:string]:TPacket read FindSub;
    property SubPackets: TPacketList read FSubPackets;  // write FSubPackets;
    property ReplyOK: bool read FReplyOK;
    property ErrorCode: string read FErrorCode;
    property Plain: string read FFields;
  end;

  TPacketList = class(TList)
  protected
    function GetItem(Index: Integer): TPacket;
    procedure SetItem(Index: Integer; Packet: TPacket);
  public
    function Add(Packet: TPacket): Integer;
    property Items[Index: Integer]: TPacket read GetItem write SetItem; default;
    procedure FreeWithSubItems;
  end;

  TUSPUser = class(TObject)
  public
    UserName,Password,Service: string;
    IP: string;
    Incoming,Outgoing: integer;
    Socket: TCustomWinSocket;
    PacketBuffer: string;
    ServiceModule: TDataModule;
    Values: string;
  end;

  TUSPUsers = class(TList)
  protected
    function GetItem(Index: Integer): TUSPUser;
    procedure SetItem(Index: Integer; User: TUSPUser);
  public
    function Add(User: TUSPUser): Integer;
    property Items[Index: Integer]: TUSPUser read GetItem write SetItem; default;
    procedure FreeWithSubItems;
  end;

const { Packets models }

  ResOK           = 'Res[OK]';
  SmallPacket     = 'Res[OK] Model[small]'+#13#10;
  ArrayPacket     = 'Res[OK] Model[large]'+#13#10+
                    'Part[ROOT] Type[array] Count[0]'+#13#10+
                    'Endpart[ROOT]'+#13#10;
  MultipartPacket = 'Res[OK] Model[large]'+#13#10+
                    'Part[ROOT] Type[multipart] Count[0]'+#13#10+
                    'Endpart[ROOT]'+#13#10;
  PacketEnd       = '.'+#13#10;

function GetStrLine(det:string;var s:string):string;
function GetStrField(field,s:string):string;
procedure SetStrField(field,value:string;var s:string);
function GetStrBlock(br:String; var s: string; del:bool): string;
function FieldEncode(s:string):string;
function FieldDecode(s:string):string;
function XSPath(long:string):string;
function SPath(long:string):string;
procedure Err(msg:string);
function IsLike(s1,s2:string):bool;
function IsInList(Item,List:string):bool;
function CharCount(s:string;c:char):integer;
function CommaCount(s:string):integer;
function CommaGetItem(a:string;n:integer):string;
procedure CommaSetItem(var a:string;n:integer;val:string);
function CommaGetField(a:string;fields:string;field:string):string;
procedure CommaSetField(var a:string;fields:string;field,val:string);
function UspToValues(s:string):string;
procedure ShowAsTable(PAK:TPacket;LV:TListView;Fields:string);
procedure ShowAsListView(PAK:TPacket;LV:TListView);
procedure ShowAsTreeView(PAK:TPacket;TV:TTreeView;TN:TTreeNode;ShowFields:bool);

implementation

{ General utils }

function Min(Const a,b: integer): integer;
begin
  if b<a then Result:=b else Result:=a;
end;

function Max(Const a,b: integer): integer;
begin
  if b<a then Result:=a else Result:=b;
end;

{ String parser }

function GetStrLine(det:string;var s:string):string;
var k:integer;
begin
  Result:='';
  k:=pos(det,s);
  if k>0 then begin
    Result:=Copy(s,1,k-1);
    Delete(s,1,k+length(det)-1);
  end;
end;

function GetStrBlock(br:String; var s: string; del:bool): string;
var k,m: integer; t:string;
begin
  t:=s;
  k:=Pos(br[1],t); if k>0 then t[k]:=' ';
  m:=Pos(br[2],t); if m>0 then t[m]:=' ';
  Result:=copy(t,k+1,m-k);
  if del then Delete(s,k,m-k+1);
end;

function GetStrField(field,s:string):string;
var k,l:integer;
begin
  Result:='';
  k:=pos(field+'[',s);
  if k>0 then begin
    inc(k,length(field)+1);
    Result:=s;
    Delete(Result,1,k-1);
    l:=pos(']',Result);
    if l>0 then Setlength(Result,l-1)
    else Result:='';
  end;
  Result:=FieldDecode(Trim(Result));
end;

procedure SetStrField(field,value:string;var s:string);
var k:integer; t:string;
begin
  k:=pos(field+'[',s);
  if k>0 then begin
    inc(k,length(field)+1);dec(k);
    t:=Copy(s,1,k); Delete(s,1,k);
    k:=pos(']',s)-1; Delete(s,1,k);
    s:=Trim(t+FieldEncode(value)+s);
  end else begin
    t:=Copy(s,length(s)-1,2);
    if t<>#13#10 then s:=s+' ';
    s:=s+field+'['+FieldEncode(value)+']';
  end;
end;

function FieldEncode(s:string):string;
var i,k:integer;
begin
  Result:='';
  for i:=1 to length(s) do begin
    k:=ord(s[i]);
    if (k<32)or(s[i] in ['$','#','(',')',',','[',']'])
    then Result:=Result+'$'+chr(k+48)
    else Result:=Result+s[i];
  end;
end;

function FieldDecode(s:string):string;
var i:integer;
begin
  Result:=''; i:=1;
  while i<=length(s) do begin
    if s[i]='$' then begin
      inc(i);
      Result:=Result+chr(ord(s[i])-48);
    end else Result:=Result+s[i];
    inc(i);
  end;
end;

function XSPath(long:string):string;
begin
  Result:=long;
  if (length(Result)>0)and(Result[length(Result)]='\')
  then Delete(Result,length(Result),1);
end;

function SPath(long:string):string;
begin
  Result:=long;
  if (length(Result)>0)and(Result[length(Result)]<>'\')
  then Result:=Result+'\';
end;

procedure Err(msg:string);
begin
  MessageBox(0,PChar(msg),PChar(Application.Title+' Error'),MB_OK or MB_ICONERROR or MB_APPLMODAL);
end;

function IsLike(s1,s2:string):bool;
var s3,s4:string; a1:dword;
begin
  Result:=false;
  if s1=s2 then Result:=true else begin
    a1:=Pos('*',s1);
    s3:=''; s4:='';
    if a1>0 then begin
      s3:=Copy(s1,1,a1-1); s4:=Copy(s2,1,a1-1);
      Delete(s1,1,a1); a1:=pos(s1,s2); Delete(s2,1,a1-1);
      if (s3=s4)and((s1=s2)or(s1='')) then Result:=true;
    end;
    a1:=Pos('?',s1);
    if (not Result)and(a1>0) then begin
      while a1>0 do begin
        s1[a1]:=s2[a1];
        a1:=Pos('?',s1);
      end;
      if s1=s2 then Result:=true;
    end;
  end;
end;

function IsInList(Item,List:string):bool;
var t:string;
begin
  t:=','+List+',';
  Result:=Pos(Item,t)>0;
end;

{ Comma parser }

function CharCount(s:string;c:char):integer;
var i:integer;
begin
  Result:=0; for i:=1 to length(s) do if s[i]=c then inc(Result);
end;

function CommaCount(s:string):integer;
begin
  Result:=CharCount(s,',');
end;

function CommaGetItem(a:string;n:integer):string;
var i:integer;
begin
  Result:=''; a:=a+','; for i:=1 to n do Result:=GetStrLine(',',a);
end;

procedure CommaSetItem(var a:string;n:integer;val:string);
var s,d:string; i:integer;
begin
  s:=''; a:=a+',';
  for i:=1 to max(CommaCount(a),n) do begin
     d:=GetStrLine(',',a); if i=n then d:=val; s:=s+d+',';
  end;
  SetLength(s,length(s)-1); a:=s;
end;

function CommaGetField(a:string;fields:string;field:string):string;
var s,f:string;
begin
  Result:=''; a:=a+','; fields:=fields+',';
  while pos(',',a)>0 do begin
    s:=GetStrLine(',',a); f:=GetStrLine(',',fields);
    if f=field then begin Result:=s; Break; end;
  end;
end;

procedure CommaSetField(var a:string;fields:string;field,val:string);
var s,f,ns:string;
begin
  ns:=''; a:=a+','; fields:=fields+',';
  while pos(',',a)>0 do begin
    s:=GetStrLine(',',a); f:=GetStrLine(',',fields);
    if f=field then begin ns:=ns+val+','+a; Break; end
    else ns:=ns+s+',';
  end;
  a:=ns;
  if a[length(a)]=',' then SetLength(a,length(a)-1);
end;

function UspToValues(s:string):string;
var d:string;
begin
  Result:='';
  while Pos(']',s)>0 do begin
    d:=Trim(GetStrLine(']',s));
    while Pos(#13#10,d)>0 do Delete(d,Pos(#13#10,s),2);
    if Pos('[',d)>0 then begin
       d[Pos('[',d)]:='=';
       Result:=Result+d+#13#10;
    end;
  end;
end;

{ Visualization }

procedure ShowAsTable(PAK:TPacket;LV:TListView;fields:string);
var LI:TListItem; i,j:integer; s,f:string; LC:TListColumn;
begin
  if (PAK<>nil)and(PAK.SubCount>0)and(LV<>nil) then begin
    LV.Items.BeginUpdate;
    if Length(Fields)=0 then begin
      f:=PAK.SubPackets[0].Plain;
      f:=GetStrBlock('[]',f,false);
      LV.Columns.Clear;
      for i:=1 to CommaCount(f)+1 do begin
        LC:=LV.Columns.Add;
        LC.Caption:=CommaGetItem(f,i);
      end;
    end;
    LV.Items.Clear;
    for i:=1 to PAK.FSubCount-1 do begin
      s:=Trim(PAK.SubPackets[i].Plain);
      s:=GetStrBlock('[]',s,false);
      LI:=LV.Items.Add;
      LI.Caption:=CommaGetItem(s,1);
      for j:=2 to CommaCount(f)+1
      do LI.Subitems.Add(CommaGetItem(s,j));
    end;
    LV.Items.EndUpdate;
  end;
end;

procedure ShowAsListView(PAK:TPacket;LV:TListView);
var LI:TListItem; i,j:integer;
begin
  if (PAK<>nil)and(LV<>nil) then begin
    LV.Items.BeginUpdate;
    LV.Items.Clear;
    for i:=0 to PAK.FSubCount-1 do begin
      LI:=LV.Items.Add;
      LI.Caption:=PAK.SubPackets[i].Fields[LV.Columns[0].Caption];
      for j:=1 to LV.Columns.Count-1
      do LI.SubItems.Add(PAK.SubPackets[i].Fields[LV.Columns[j].Caption]);
    end;
    LV.Items.EndUpdate;
  end;
end;

procedure ShowAsTreeView(PAK:TPacket;TV:TTreeView;TN:TTreeNode;ShowFields:bool);
var i:integer; s:string;
begin
  if (PAK<>nil)and(not (TV=nil)or(TN=nil)) then begin
    if TV=nil then TV:=TTreeView(TN.TreeView);
    if TN=nil then begin
      TV.Items.Clear;
      TN:=TV.Items.AddChild(nil,'');
    end else TN:=TV.Items.AddChild(TN,'');
    TN.Data:=PAK; s:=PAK.FFields;
    if PAK.PacketName<>'' then begin
      TN.Text:=PAK.PacketName;
      //if s<>'' then TV.Items.AddChild(TN,s);
      if ShowFields then while Pos(#13#10,s)>0 do TV.Items.AddChild(TN,Trim(GetStrLine(#13#10,s)));
    end else begin
      s:=PAK.FFields;
      s:=GetStrBlock('[]',s,false);
      TN.Text:=Trim(s);
    end;
    for i:=0 to PAK.SubCount-1 do ShowAsTreeView(PAK.SubPackets[i],TV,TN,ShowFields);
  end;
end;

{ TPacket }

constructor TPacket.Create(pText: string);
begin
  inherited Create;
  FText:=ptext;
  DecodePacket;
end;

destructor TPacket.Destroy;
begin
  if FSubPackets<>nil then FSubPackets.FreeWithSubItems;
  inherited Destroy;
end;

procedure TPacket.CreateSubPacket(pText: string);
begin
  Inc(FSubCount);
  SetStrField('Count',IntToStr(FSubCount),FText);
  if FSubPackets=nil then FSubPackets:=TPacketList.Create;
  FSubPackets.Add(TPacket.Create(pText));
  //ShowMessage(FText);
end;

function TPacket.GetField(name: string):string;
begin
  Result:=GetStrField(name,FFields);
end;

procedure TPacket.SetField(name,value: string);
begin
  SetStrField(name,value,FFields);
end;

function TPacket.GetCell(row,col:integer):string;
var s:string; i:integer;
begin
  Result:='';
  if Subpackets[row]<>nil then begin
    s:=Subpackets[row].Text;
    s:=GetStrBlock('[]',s,false);
    for i:=1 to col do Result:=GetStrLine(',',s);
    Result:=FieldDecode(Result);
  end;
end;

procedure TPacket.SetCell(row,col:integer;val:string);
begin
  {!!!}
end;

function TPacket.GetFieldCell(row:integer;field:string):string;
var s,f:string;
begin
  Result:='';
  if (PacketType=ptTable)and(Subpackets[row]<>nil) then begin
    s:=Subpackets[row].Text; s:=GetStrBlock('[]',s,false);
    f:=Subpackets[0].Text; f:=GetStrBlock('[]',f,false);
    Result:=CommaGetField(s,f,field);
    Result:=FieldDecode(Result);
  end;
end;

procedure TPacket.SetFieldCell(row:integer;field,val:string);
var s,f:string;
begin
  if (PacketType=ptTable)and(Subpackets[row]<>nil) then begin
    s:=Subpackets[row].Text; s:=GetStrBlock('[]',s,false);
    f:=Subpackets[0].Text; f:=GetStrBlock('[]',f,false);
    CommaSetField(s,f,field,FieldEncode(val));
    Subpackets[row].Text:='['+s+']';
  end;
end;

function TPacket.FindSub(name: string):TPacket;
var i:integer;
begin
  Result:=nil; i:=0;
  while (i<FSubCount)and(Result=nil) do begin
    if FSubPackets[i].PacketName=name
    then Result:=FSubPackets[i];
    inc(i);
  end;
end;

function TPacket.ClipText:string;
var i:integer;
begin
  Result:='';
  if ErrorCode<>'' then begin
    Result:='Res['+ErrorCode+'] ';
    if Self.Model=pmLarge
    then Result:=Result+'Model[large]'
    else Result:=Result+'Model[small]';
    Result:=Result+#13#10;
  end;
  if FPacketName='' then Result:=FFields
  else begin
    Result:=Result+'Part['+FPacketName+'] Type[';
    case PacketType of
      ptPlain:     Result:=Result+'plain';
      ptMultipart: Result:=Result+'multipart';
      ptArray:     Result:=Result+'array';
      ptFile:      Result:=Result+'file';
      ptTable:     Result:=Result+'table';
    end;
    Result:=Result+']';
    if (PacketType=ptMultipart)or(PacketType=ptArray)
    then Result:=Result+' Count['+IntToStr(SubCount)+']';
    Result:=Result+#13#10;
    if FFields<>'' then begin
      Result:=Result+FFields;
      if PacketType=ptMultipart
      then Result:=Result+#13#10;
    end;
    if (PacketType=ptMultipart)or(PacketType=ptArray) then  begin
      for i:=0 to SubCount-1 do begin
        Result:=Result+SubPackets[i].Text+#13#10;
      end;
    end;
    Result:=Result+'Endpart['+FPacketName+']';
  end;
end;

procedure TPacket.DecodePacket;
var pak,s,t,d:string;
    i:integer;
begin
  pak:=FText;
  s:=GetStrLine(#13#10,pak);              // Main Header
  t:=GetStrField('Res',s);
  if t<>'' then begin
    FErrorCode:=t;
    FReplyOK:=(t='OK');
    if GetStrField('Model',s)='large'
    then FPacketModel:=pmLarge
    else FPacketModel:=pmSmall;
  end else begin                          // SubPacket Header
    FPacketModel:=pmLarge;
    pak:=FText;
  end;
  if FPacketModel=pmSmall then begin      // Small Packet
    FFields:=pak;
    FPacketName:='none';
    FType:=ptPlain;
    FSubCount:=0;
  end else begin
    s:=GetStrLine(#13#10,pak);            // Large Packet Header
    FPacketName:=GetStrField('Part',s);
    t:=GetStrField('Type',s);
    FType:=ptPlain;
    if t='multipart' then FType:=ptMultipart;
    if t='array' then FType:=ptArray;
    if t='file' then FType:=ptFile;
    if t='table' then FType:=ptTable;
    if (FType=ptPlain)or(FType=ptFile) then begin
      if pos('Endpart['+FPacketName+']',pak)>0
      then pak:=GetStrLine('Endpart['+FPacketName+']',pak);
      FFields:=pak;
    end;
    if FType=ptMultipart then begin       // Multipart/Recursive Sub Packet
      FSubCount:=StrToInt(GetStrField('Count',s));
      FSubPackets:=TPacketList.Create;
      for i:=1 to FSubCount do begin
        t:=GetStrField('Part',pak);
        d:=Trim(GetStrLine('Part['+t+']',pak));
        if d<>'' then FFields:=d;
        pak:='Part['+t+']'+pak;
        s:=GetStrLine('Endpart['+t+']'+#13#10,pak);
        FSubPackets.Add(TPacket.Create(s+'Endpart['+t+']'));
      end;
    end;
    if (FType=ptArray)or(FType=ptTable) then begin // Array or Table Sub Packet
      FSubCount:=StrToInt(GetStrField('Count',s));
      FSubPackets:=TPacketList.Create;
      for i:=1 to FSubCount do begin
        s:=GetStrLine(#13#10,pak);
        FSubPackets.Add(TPacket.Create(s));
      end;
    end;
  end;
end;

{ TPacketList }

function TPacketList.Add(Packet: TPacket): Integer;
begin
  Result:=inherited Add(Packet);
end;

function TPacketList.GetItem(Index: Integer): TPacket;
begin
  Result:=inherited Items[Index];
end;

procedure TPacketList.SetItem(Index: Integer; Packet: TPacket);
begin
  inherited Items[Index]:=Packet;
end;

procedure TPacketList.FreeWithSubItems;
var i: integer;
begin
  for i:=0 to Count-1 do
  TPacket(Items[I]).Free;
  Free;
end;

{ TUSPUsers }

function TUSPUsers.Add(User: TUSPUser): Integer;
begin
  Result:=inherited Add(User);
end;

function TUSPUsers.GetItem(Index: Integer): TUSPUser;
begin
  Result:=inherited Items[Index];
end;

procedure TUSPUsers.SetItem(Index: Integer; User: TUSPUser);
begin
  inherited Items[Index]:=User;
end;

procedure TUSPUsers.FreeWithSubItems;
var i: integer;
begin
  for i:=0 to Count-1 do
  TUSPUser(Items[I]).Free;
  Free;
end;

end.