unit UnitSHFS;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  USPFunction, USPUtils, USPServer, FileCtrl, tlps, Registry;

type
  TSHFS = class(TDataModule)
    List: TUSPFunction;
    FileOp: TUSPFunction;
    procedure ListCall(User: TUSPUser; Par: TStrings;
      var Result: TStrings);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  const
    ROOTMax = 10;
    REGRMax = 6;
    hex: string = '0123456789ABCDEF';
    s16: array [0..1] of byte = (1,16);
  type
    TROOT = Array[0..ROOTMax] of string;
    TREGR = Array[0..REGRMax] of string;

    PNetResourceArray = ^TNetResourceArray;
    TNetResourceArray = array[0..MaxInt div SizeOf(TNetResource) - 1] of TNetResource;

var
  SHFS: TSHFS;
  OSV: TOSVersionInfoA;
  MA: array [0..63] of Cardinal;    
  MINF: MODULEINFO;
  Reg: TRegistry;

implementation

const

  ROOT:TROOT = ('Disks','Network','Windows','Processes','Events','Registry','Devices',
                'Menu','Setup','Desktop','Programs');
  REGR:TREGR = ('HKEY_CLASSES_ROOT','HKEY_CURRENT_USER',
                'HKEY_LOCAL_MACHINE','HKEY_USERS',
                'HKEY_PERFORMANCE_DATA','HKEY_CURRENT_CONFIG',
                'HKEY_DYN_DATA');

{$R *.DFM}

function Byte2Hex(a:byte):string;
begin
  Result:=hex[a mod 16+1]+hex[(a div 16) mod 16+1];
end;

function Hex2Byte(s:string):byte;
var i,j:byte;
begin
  Result:=0;
  for i:=1 to 2 do begin
    for j:=1 to 16 do
    if s[i]=hex[j] then Result:=Result+(j-1)*s16[i-1];
  end;
end;

function Str2Hex(s:string):string;
var i:integer;
begin
  Result:='';
  for i:=1 to length(s)
  do Result:=Result+Byte2Hex(Ord(s[i]))+' ';
end;

function Hex2Str(s:string):string;
var i:integer; e,d:string;
begin
  Result:=''; e:=s;
  i:=Pos(' ',e);
  repeat
    d:=copy(e,1,i-1);
    delete(e,1,i);
    Result:=Result+Chr(Hex2Byte(d));
    i:=Pos(' ',e);
  until i>0;
end;

function RGB2Dword(s:string):dword;
begin
  s:=UpperCase(s);
  Result:=RGB(Hex2Byte(s[2]+s[1]),Hex2Byte(s[4]+s[3]),Hex2Byte(s[6]+s[5]));
end;

function IsCharNum(C: Char):Bool;
var b: byte;
begin
  b:=ord(c);
  if (b=45)or((b>47)and(b<58))
  then Result:=true
  else Result:=false;
end;

function Str2Int(S: string):Integer;
var i: integer;
begin
  Result:=0;
  for i:=1 to length(S) do if not IsCharNum(S[i]) then Result:=-1;
  if length(S)=0 then Result:=-1;
  if Result<>-1 then Result:=StrToInt(s);
end;

{---}

function GetRootIndex(s:string):integer;
var i:integer;
begin
  Result:=-1; i:=0;
  while (s<>ROOT[i])and(i<ROOTMax+1) do inc(i);
  if s=ROOT[i] then Result:=i;
end;

function GetRegrIndex(s:string):integer;
var i:integer;
begin
  Result:=-1; i:=0;
  while (s<>REGR[i])and(i<REGRMax+1) do inc(i);
  if s=REGR[i] then Result:=i;
end;

function SHFS_GetShort(long:string):string;
var k:integer;
begin
  Result:=long;
  k:=Pos('\',Result); if k>0 then Result[k]:='#';
  k:=Pos('\',Result); if k>0 then Delete(Result,1,k)
  else Result:='';
end;

function SHFS_GetSubRoot(long:string):string;
var s:string;
begin
  s:=long+'\'; Result:=GetStrBlock('\\',s,false);
end;

function SHFS_MakePath(path,subdir:string):string;
begin
  if subdir='..' then begin
    Result:=path;
    if Result[length(Result)]=':' then Delete(Result,length(Result),1);
    Result:=XSPath(ExtractFilePath(Result));
  end else if subdir[1]<>'\' then Result:=path+'\'+subdir
  else Result:=path+subdir;
end;

function SHFS_ExtractID(long:string):integer;
var s:string;
begin
  s:=ExtractFileName(long);
  Result:=Str2Int(GetStrBlock('()',s,false));
  if Result=-1 then Result:=0;
end;

function XSPath(long:string):string;
begin
  Result:=long;
  if (length(Result)>0)and(Result[length(Result)]='\')
  then Delete(Result,length(Result),1);
end;

function SPath(long:string):string;
begin
  Result:=long;
  if (length(Result)>0)and(Result[length(Result)]<>'\')
  then Result:=Result+'\';
end;

function GetPathLevel(long:string):integer;
var s:string; k:integer;
begin
  Result:=0;
  s:=long;
  repeat
    k:=Pos('\',s);
    if k>0 then begin
      s[k]:='@';
      inc(Result);
    end;
  until k=0
end;

function SizeToStr(size:longint):string;
begin
  Result:='decel';
  if lo(size)>1024 then Result:=IntToStr(Round(size/1024))+' Kb';
  if lo(size)>1048576 then Result:=IntToStr(Round(size/1048576))+' Mb';
  if (lo(size)>1073741824)or(size<0) then Result:=IntToStr(Round(size/1073741824))+' Gb';
end;

{procedure AddCol(List:TListView;s:string;w:integer);
var Col:TListColumn;
begin
  Col:=List.Columns.Add;
  Col.Caption:=s;
  Col.Width:=w;
end;

procedure AddItem(List:TListView;s,t:string);
var Item:TListItem;
begin
  Item:=List.Items.Add;
  Item.Caption:=s;
  Item.SubItems.Text:=t;
end;}

procedure FillFileList(List:TStrings; FDirectory:String;isMnt:bool);
var AttrIndex: TFileAttr;
  FileType: TFileType;
  FMask,FileExt: string;
  i,j: Integer;
  MaskPtr: PChar;
  Ptr: PChar;
  AttrWord: Word;
  FileInfo: TSearchRec;
  SaveCursor: TCursor;
  Mnt: TStringList;
const
   Attributes: array[TFileAttr] of Word = (faReadOnly, faHidden, faSysFile,
     faVolumeID, faDirectory, faArchive, 0);
begin
  if not DirectoryExists(FDirectory) then begin
    List.Add('Name[unmounted]'); Exit;
  end;
  FileType:=[ftDirectory,ftReadOnly,ftHidden,ftSystem,ftArchive,ftNormal];
  FMask:='*.*';
  AttrWord:=0;
  for AttrIndex:=ftReadOnly to ftArchive
  do if AttrIndex in FileType then AttrWord:=AttrWord or Attributes[AttrIndex];
  ChDir(FDirectory);
  I:=0;
  SaveCursor:=Screen.Cursor;
  try
    MaskPtr:=PChar(FMask);
    while MaskPtr<>nil do begin
      Ptr:=StrScan(MaskPtr,';');
      if Ptr<>nil then Ptr^:=#0;
      if FindFirst(MaskPtr,AttrWord,FileInfo)=0 then begin
        repeat
          if (ftNormal in FileType)or(FileInfo.Attr and AttrWord<>0) then
            if FileInfo.Attr and faDirectory<>0 then begin
              if (FileInfo.Name<>'.')and(FileInfo.Name<>'..')
              then List.Add('Name[\'+UpperCase(FileInfo.Name)+']');
              Inc(I);
            end else begin
              FileExt:=AnsiLowerCase(ExtractFileExt(FileInfo.Name));
              if (isMnt)and(LowerCase(FileInfo.Name)='shfs.mnt') then begin
                mnt:=TStringList.Create;
                mnt.LoadFromFile('shfs.mnt');
                for j:=0 to mnt.Count-1 do FillFileList(List,mnt[j],isMnt);
                mnt.Free;
              end else List.Add('Name['+LowerCase(FileInfo.Name)+'] Size['+IntToStr(FileInfo.Size)+']');
              Inc(I);
            end;
          if I=50 then
            Screen.Cursor:=crHourGlass;
        until FindNext(FileInfo)<>0;
        FindClose(FileInfo);
      end;
      if Ptr<>nil then begin
        Ptr^:=';';
        Inc(Ptr);
      end;
      MaskPtr:=Ptr;
    end;
  finally
    Screen.Cursor:=SaveCursor;
  end;
end;

function GetPriorityName(PR:integer):string;
begin
  case PR of
    NORMAL_PRIORITY_CLASS,8    : Result:='Normal';
    IDLE_PRIORITY_CLASS,4      : Result:='Idle';
    HIGH_PRIORITY_CLASS,13     : Result:='High';
    REALTIME_PRIORITY_CLASS,24 : Result:='Realtime';
  end;
end;

procedure FillTaskList(List:TStrings);
type TBuf = array [0..0] of char;
var snap:Thandle;
    HND,PR: integer; B:Bool; spr: string;
    buff: array [0..511] of char;
    i,pid,needed: integer;
    s: string;
    Process: ^TProcessEntry32;
  hLib, Cur : integer;
  Ptr  : Pointer;
  PrcInfo : PProcessInfo;
  Buf : ^TBuf;
begin
  if OSV.dwPlatformId=2 then begin
    GetMem(Buf,20480);
    NTQuerySystemInformation(5,Pointer(Buf),20480,0);
    Cur:=0; Ptr:=@Buf[Cur];
    PrcInfo:=Ptr;
    repeat
      Inc(Cur,PrcInfo.dwOffset);
      Ptr:=@Buf[Cur];
      PrcInfo:=Ptr;
      List.Add('Name[\'+ExtractFilename(PrcInfo^.pszProcessName)+
                   '('+IntToStr(PrcInfo^.dwProcessID)+')] Priority['+GetPriorityName(PrcInfo^.dwBasePriority)+']');
    until PrcInfo.dwOffset=0;
    FreeMem(Buf,20480);
{    EnumProcesses(@PA,64*4,needed);
    for i:=0 to Round(needed/4)-1 do begin
      pid:=PA[i];
      hnd:=OpenProcess(PROCESS_QUERY_INFORMATION+PROCESS_VM_READ,true,pid);
      GetModuleFileNameEx(hnd,0,@buff,512);
      s:=buff;
      PR:=GetPriorityClass(HND);
      CloseHandle(hnd);
      spr:=GetPriorityName(PR);
      //GetProcessTimes(HND,PCT,PET,PKT,PUT);
      AddItem(List,'\'+ExtractFilename(s)+'('+IntToStr(pid)+')',
              spr+#13#10+'');
    end;}
  end else begin
    GetMem(Process,SizeOf(TProcessEntry32));
    snap:=CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
    B:=Process32First(snap,Process^);
    repeat
      if B then begin
        HND:=OpenProcess(PROCESS_ALL_ACCESS,true,Process^.th32ProcessID);
        PR:=GetPriorityClass(HND); CloseHandle(HND);
        spr:=GetPriorityName(PR);
        List.Add('Name[\'+ExtractFilename(Process^.szExeFile)+
                '('+IntToStr(Process^.th32ProcessID)+')] Priority['+spr+']');
                //#13#10+IntToStr(Process^.cntUsage)+'%');
      end;
      B:=Process32Next(snap,Process^);
    until not B;
    CloseHandle(snap);
    FreeMem(Process,SizeOf(TProcessEntry32));
  end;
end;

procedure FillModuleList(List:TStrings;PID:integer);
var snap:Thandle; B:Bool;
    i,hnd,needed: integer;
    buff: array [0..127] of char;
    s: string;
    Module:^TModuleEntry32;
begin
  if OSV.dwPlatformId=2 then begin
    hnd:=OpenProcess(PROCESS_QUERY_INFORMATION+PROCESS_VM_READ,true,pid);
    EnumProcessModules(hnd,@MA,64*4,Cardinal(needed));
    for i:=0 to Round(needed/4)-1 do begin
      GetModuleInformation(hnd,MA[i],@MINF,SizeOf(MODULEINFO));
      GetModuleBaseName(hnd,MA[i],@buff,128);
      s:=buff;
      List.Add('Name['+ExtractFileName(s)+'] Size['+IntToStr(MINF.SizeOfImage)+
             '] Handle['+IntToStr(MA[i])+'] ID['+IntToStr(0)+']');
    end;
    CloseHandle(hnd);
  end else begin
    GetMem(Module,SizeOf(TModuleEntry32));
    snap:=CreateToolhelp32Snapshot(TH32CS_SNAPALL,PID);
    B:=Module32First(snap,Module^);
    repeat
      if B then begin
        List.Add('Name['+Module^.szModule+'] Size['+IntToStr(Module^.modBaseSize)+
               '] Handle['+IntToStr(Module^.hModule)+'] ID['+IntToStr(Module^.th32ModuleID)+']');
      end;
      B:=Module32Next(snap,Module^)
    until not B;
    CloseHandle(snap);
    FreeMem(Module,SizeOf(TModuleEntry32));
  end;
end;

procedure FillWinList(List:TStrings;hOWnd:integer;tVisible:bool);
var Wnd: hWnd; buff: array [0..127] of Char; t,v:string;
begin
  Wnd:=GetWindow(Application.Handle,GW_HWNDFIRST);
  while Wnd<>0 do begin
    if (IsWindowVisible(Wnd)or(not tVisible))and
       (GetWindow(Wnd,GW_OWNER)=hOWnd)and
       (GetWindowText(Wnd,buff,sizeof(buff))<>0)
    then begin
      t:=buff; v:=' '; if IsWindowVisible(Wnd) then v:='';
      GetClassName(Wnd,buff,sizeof(buff));
      List.Add('Name(Handle)[\'+t+'('+IntToStr(wnd)+')] Class['+buff+'] V['+v+']');
    end;
    Wnd:=GetWindow(Wnd,GW_HWNDNEXT);
  end;
  Wnd:=GetWindow(hOWnd,GW_HWNDFIRST+GW_CHILD);
  while Wnd<>0 do begin
    GetWindowText(Wnd,buff,sizeof(buff));
    t:=buff; v:=' '; if IsWindowVisible(Wnd) then v:='';
    GetClassName(Wnd,buff,sizeof(buff));
    List.Add('Hame(Handle)[\'+t+'('+IntToStr(wnd)+')] Class['+buff+'] V['+v+']');
    Wnd:=GetNextWindow(Wnd,GW_HWNDNEXT);
  end;
end;

procedure FillJumpList(List:TStrings;short:string);
var i,k:integer; TMPFile:TStrings; s,d,ofs:string;
begin
  TMPFile:=TStringlist.Create;
  TMPFile.LoadFromFile(CurDir+'usermenu.vfl');
  for i:=1 to TMPFile.Count-1 do {if Pos(short,TMPFile[i])=1 then} begin
    s:=TMPFile[i];
    if s[1]<>';' then begin
      List.Add('Name['+GetStrBlock('<>',s,false)+'] Caption['+GetStrBlock('[]',s,false)+']');
    end;
  end;
  TMPFile.Free;
end;

procedure FillNetList(List:TStrings;short:string); //lpNetResource: PNetResource;
var
  i,j,l,NetResult,k: Integer; Drive: Char;
  E,NetHandle,Count,BufSize,Size: integer;
  NetResources: PNetResourceArray;
  RemoteNameInfo: array[0..1023] of Byte;
  NR: PNetResource; s,d,r,t:string;
  CFG: TStrings;
begin
  GetMem(NR,SizeOf(TNetResource));
  NR^.dwScope:=RESOURCE_GLOBALNET;
  NR^.dwType:=RESOURCETYPE_ANY;
  s:=short; l:=Pos('\',s);
  if l>0 then d:=Copy(s,1,l-1) else d:=s;
  NR^.lpProvider:=PChar(d);
  NR^.dwUsage:=RESOURCEUSAGE_CONTAINER;
  s:=ExtractFilename(Short);
  k:=GetPathLevel(short);

  if k>1 then s:='\\'+s;
  if s=d then s:=''+#0;
  NR^.lpRemoteName:=PChar(s);
  if short='' then begin
    E:=WNetOpenEnum(RESOURCE_GLOBALNET,RESOURCETYPE_ANY,0,nil,Cardinal(NetHandle));
    List.Add('Name[\MWN] Description[Network]');
    List.Add('Name[\Icmp] Description[Ping]');
    List.Add('Name[\FTP] Description[Port 21]');
    List.Add('Name[\SHFS] Description[Shaman]');
  end else begin
    NR^.dwUsage:=RESOURCEUSAGE_CONTAINER;
    if (GetPathLevel(short)=0)and(OSV.dwPlatformId=2)
    then NR^.dwUsage:=NR^.dwUsage+RESOURCEUSAGE_RESERVED;
    E:=WNetOpenEnum(RESOURCE_GLOBALNET,RESOURCETYPE_ANY,0,NR,Cardinal(NetHandle));
  end;
  if (short='MWN')or(short='Icmp')or(short='FTP')or(short='SHFS') then begin
    CFG:=TStringlist.Create;
    CFG.LoadFromFile(CurDir+'shaman.net');
    for i:=0 to CFG.Count-1 do begin
      t:=UpperCase(CFG[i]); r:=t;
      l:=Pos('\\',r);
      if l>0 then Delete(r,1,l+1);
      setlength(t,length(short));
      if t='MWN' then List.Add('Name[\'+r+']');
    end;
    //AddItem(List,'unmounted','');
    E:=NO_ERROR+10;
    CFG.Free;
  end;
  if E=NO_ERROR then begin
    BufSize:=16*SizeOf(TNetResource);
    GetMem(NetResources,BufSize);
    while True do begin
      Count:=-1;
      Size:=BufSize;
      NetResult:=WNetEnumResource(NetHandle,Cardinal(Count),NetResources,Cardinal(Size));
      if NetResult=ERROR_MORE_DATA then begin
        BufSize:=Size;
        ReallocMem(NetResources,BufSize);
        Continue;
      end;
      if NetResult<>NO_ERROR then begin
        Exit;
      end;
      for I:=0 to Count-1 do with NetResources^[I] do begin
        s:=lpRemoteName;
        if Pos('\\',s)=1 then Delete(s,1,2);
        l:=Pos('\',s);
        if l>0 then Delete(s,1,l);
        if s='' then s:=lpComment;
        //if (dwUsage and RESOURCEUSAGE_CONTAINER)<>0 then
        List.Add('Name[\'+s+'] Description['+lpComment+']');
      end;
    end;
    FreeMem(NetResources,BufSize);
  end;
  WNetCloseEnum(NetHandle);
  FreeMem(NR,SizeOf(TNetResource));
end;

procedure FillDevList(List:TStrings;short:string);
var DA: array [0..511] of Pointer;
    buff: array [0..127] of char;
    Needed,i: integer; s: string;
begin
{  EnumDeviceDrivers(@DA,512,Needed);
  for i:=0 to Round(needed/4)-1 do begin
    GetDeviceDriverFileName(DA[i],@buff,128);
    //GetDeviceDriverBaseName(DA[i],@buff,128);
    s:=buff; s:=ExtractFileName(s);
    AddItem(List,s,'');
  end;}
end;

procedure TSHFS.ListCall(User: TUSPUser; Par: TStrings; var Result: TStrings);
//procedure List(path:string;List:TListView);
var di,short,subroot,regsub,regshort,part,path:string;
    i,k,l,DriveType:integer; FTC,TNB,NFB:Int64;
    Drives: set of 0..25; TMPFile: TStrings;
    buff: array [0..127] of Char;
begin
  path:=par[0];
  Result.Add('Res[OK] Model[large]');
  Result.Add('Part[ls] Type[multipart] Count[2]');
  Result.Add('Part[Header] Type[array] Count[0]');
  if path='ROOT' then begin
    Result.Add('Name[Name] Size[150]');
    Result.Add('Endpart[Header]');
    for i:=0 to ROOTMax do Result.Add('Name[\'+ROOT[i]+']');
  end else begin
    short:=SHFS_GetShort(path);
    subroot:=SHFS_GetSubRoot(path);
    i:=GetRootIndex(subroot);
    Result.Add('Name[..]');
    l:=GetPathLevel(short);
    if (i=1)and(l>2) then begin
      l:=Pos('\',short); if l>0 then Delete(short,1,l);
      l:=Pos('\',short); if l>0 then Delete(short,1,l);
      short:='\\'+short;
      i:=0;
    end;
    //ShowInfo(short,'');
    case i of
      0: begin {Disks}
           if short='' then begin
             Result.Add('Name[Name] Size[50]');
             Result.Add('Name[Type] Size[70]');
             Result.Add('Name[Size] Size[60]');
             Result.Add('Name[Free] Size[60]');
             Integer(Drives):=GetLogicalDrives;
             for k:=0 to 25 do if k in Drives then begin
               DriveType:=GetDriveType(PChar(Chr(k+65)+':\'));
               di:='';
               case DriveType of
               DRIVE_REMOTE: di:='Type[Network] ';
               DRIVE_RAMDISK: di:='Type[RAM disk] ';
               DRIVE_FIXED: di:='Type[Hard] ';
               DRIVE_UNKNOWN: di:='Type[Unknown] ';
               DRIVE_NO_ROOT_DIR: di:='Type[No ROOT] ';
               DRIVE_REMOVABLE: di:='Type[Floppy] ';
               DRIVE_CDROM: di:='Type[CDROM] ';
               end;
               if DriveType in [DRIVE_RAMDISK,DRIVE_FIXED] then begin
                 GetDiskFreeSpaceEx(PChar(Chr(k+65)+':\'),FTC,TNB,@NFB);
                 di:=di+' Size['+SizeToStr(TNB)+'] Free['+SizeToStr(NFB)+']';
               end;
               Result.Add('Name[\'+Chr(k+65)+':] '+di);
             end;
           end else begin
             Result.Add('Name[Name] Size[120]');
             Result.Add('Name[Size] Size[70]');
             if short[length(short)]=':' then short:=SPath(short);
             FillFileList(Result,short,true{Form1.RvLog.Checked});
             //List.AlphaSort;
           end;
         end;
      1: begin {Network}
           Result.Add('Name[Name] Size[200]');
           Result.Add('Name[Description] Size[80]');
           FillNetList(Result,short);
         end;
      2: begin {Windows}
           Result.Add('Name[Name(Handle)] Size[200]');
           Result.Add('Name[Class] Size[80]');
           Result.Add('Name[V] Size[30]');
           FillWinList(Result,SHFS_ExtractID(short),false);
         end;
      3: begin {Processes}
           if short='' then begin
             Result.Add('Name[Name(ID)] Size[200]');
             Result.Add('Name[Priority] Size[80]');
             Result.Add('Name[KrnTime] Size[50]');
           end else begin
             Result.Add('Name[Name] Size[120]');
             Result.Add('Name[Size] Size[80]');
             Result.Add('Name[Handle] Size[60]');
             Result.Add('Name[ID] Size[50]');
             FillModuleList(Result,SHFS_ExtractID(short));
           end;
         end;
      4: begin {Events}
           Result.Add('Name[Time] Size[50]');
           Result.Add('Name[Command] Size[200]');
         end;
//       5: begin {Registry}
{           Result.Add('Name[Name] Size[150]');
           Result.Add('Name[Type] Size[40]');
           Result.Add('Name[Value] Size[200]');
           if short='' then begin
             for i:=0 to REGRMax do Result.Add('Name[\'+REGR[i]+']');
           end else begin
             regsub:=Shfs_GetSubRoot('\'+short);
             regshort:=Shfs_GetShort('\'+short);
             Reg.RootKey:=HKEY_CLASSES_ROOT+GetRegrIndex(regsub);
             Reg.OpenKey(regshort,false);
             TMPFile:=TStringList.Create;
             Reg.GetKeyNames(TMPFile);
             for i:=0 to TMPFile.Count-1 do AddItem(List,'\'+TMPFile[i],'key');
             Reg.GetValueNames(TMPFile);
             for i:=0 to TMPFile.Count-1 do begin
               case Reg.GetDataType(TMPFile[i]) of
                 rdString: AddItem(List,TMPFile[i],'STR'+#13#10+Reg.ReadString(TMPFile[i]));
                 rdInteger: AddItem(List,TMPFile[i],'INT'+#13#10+IntToStr(Reg.ReadInteger(TMPFile[i])));
                 rdBinary: begin
                   Reg.ReadBinaryData(TMPFile[i],buff,SizeOf(buff));
                   AddItem(List,TMPFile[i],'HEX'+#13#10+Str2Hex(buff));
                 end;
               else
                 AddItem(List,TMPFile[i],'???');
               end;
             end;
             TMPFile.Free;
             Reg.CloseKey;
           end;
         end;}
      6: begin {Devices}
           Result.Add('Name[Name] Size[150]');
           Result.Add('Name[Type] Size[60]');
           FillDevList(Result,short);
         end;
      7: begin {'Menu'}
           Result.Add('Name[Name] Size[60]');
           Result.Add('Name[Caption] Size[200]');
           FillJumpList(Result,short);
         end;
      8: begin {'Setup'}
         end;
//      9: begin {'Desktop'}
{           Result.Add('Name[Name] Size[200]');
           Result.Add('Name[Type] Size[60]');
           FillFileList(Result,DeskDir+'\'+short,Form1.RvLog.Checked);
           List.AlphaSort;
         end;}
//      10:begin {'Programs'}
{           Result.Add('Name[Name] Size[200]');
           Result.Add('Name[Type] Size[60]');
           FillFileList(Result,ProgDir+'\'+short,Form1.RvLog.Checked);
           List.AlphaSort;
         end;}
    end;
  end;
  Result.Add('Endart[List]');
  Result.Add('Endpart[ls]');
//  Result[2]:=SetStrField('Count[',']',1
end;

end.
